--------------------------------------------------------
-- CS 535 - Snort on a Chip
-- Author: Michael Attig (mea1@arl.wustl.edu)
-- Created: July 31st, 2003
-- File Name: snort_app.vhd
-- File Description:  This is the module that sits inside
--  the wrappers so that it can process packets. 
--------------------------------------------------------

LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL ;

ENTITY snort_app IS
  PORT (   
    clk                 : IN STD_LOGIC ;
    reset_l             : IN STD_LOGIC ;
    
    -- from wrapper ingress (pre-module)
    D_OUT_APPL          : IN STD_LOGIC_VECTOR(31 downto 0) ;
    DataEn_OUT_APPL     : IN STD_LOGIC ;
    SOF_OUT_APPL        : IN STD_LOGIC ;
    SOIP_OUT_APPL       : IN STD_LOGIC ;
    EOF_OUT_APPL        : IN STD_LOGIC ;
    SOP_OUT_APPL        : IN STD_LOGIC ;
    TCA_OUT_APPL        : OUT STD_LOGIC ;
    
    -- to wrapper egress (post-module)
    D_APPL_IN           : OUT STD_LOGIC_VECTOR(31 downto 0) ;
    DataEn_APPL_IN      : OUT STD_LOGIC ;
    SOF_APPL_IN         : OUT STD_LOGIC ;
    SOIP_APPL_IN        : OUT STD_LOGIC ;
    EOF_APPL_IN         : OUT STD_LOGIC ;
    SOP_APPL_IN         : OUT STD_LOGIC ;
    TCA_APPL_IN         : IN STD_LOGIC  ;
    
    -- Will use SRAM signals later
    -- To SRAM Device 0
    SRAM_Addr_dev0      : OUT STD_LOGIC_VECTOR(17 DOWNTO 0) ;
    SRAM_Data_dev0      : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0) ;
    SRAM_gw_dev0        : OUT STD_LOGIC ;
    SRAM_ce_dev0        : OUT STD_LOGIC ;
    SRAM_pin84_dev0     : OUT STD_LOGIC ;
-- To SRAM Device 1
    SRAM_Addr_dev1      : OUT STD_LOGIC_VECTOR(17 DOWNTO 0) ;
    SRAM_Data_dev1      : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0) ;
    SRAM_gw_dev1        : OUT STD_LOGIC ;
    SRAM_ce_dev1        : OUT STD_LOGIC ;
    SRAM_pin84_dev1     : OUT STD_LOGIC     

    );   
END snort_app ;

ARCHITECTURE sa OF snort_app IS

-------------------------------------------------------------------------------
-- Component Declarations 
-------------------------------------------------------------------------------

COMPONENT cpp_rules_programmer
  PORT ( clk                    : IN STD_LOGIC ;
         reset_l                : IN STD_LOGIC ;

         -- from wrappers
         dataen_out_appl_int    : IN STD_LOGIC ;
         sof_out_appl_int       : IN STD_LOGIC ;
         soip_out_appl_int      : IN STD_LOGIC ;
         eof_out_appl_int       : IN STD_LOGIC ;
         sop_out_appl_int       : IN STD_LOGIC ;
         d_out_appl_int         : IN STD_LOGIC_VECTOR(31 DOWNTO 0) ;
         tca_out_appl_int       : OUT STD_LOGIC 
         );
END COMPONENT ;

COMPONENT alert_generator
  PORT (
    clk                         : IN STD_LOGIC ;
    reset_l                     : IN STD_LOGIC ;
    
    -- from statistics block
    counter_data                : IN STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    counter_data_strobe         : IN STD_LOGIC ;
    
    -- to wrappers
    tca_appl_in_int             : IN STD_LOGIC ;
    d_appl_in_int               : OUT STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    dataen_appl_in_int          : OUT STD_LOGIC ;
    sof_appl_in_int             : OUT STD_LOGIC ;
    soip_appl_in_int            : OUT STD_LOGIC ;
    eof_appl_in_int             : OUT STD_LOGIC ;
    sop_appl_in_int             : OUT STD_LOGIC
    );
END COMPONENT ;

-------------------------------------------------------------------------------
-- Signal Declarations
-------------------------------------------------------------------------------

-- from wrappers
SIGNAL dataen_out_appl_int, sof_out_appl_int  : STD_LOGIC ;
SIGNAL soip_out_appl_int, eof_out_appl_int    : STD_LOGIC ;
SIGNAL sop_out_appl_int, tca_out_appl_int     : STD_LOGIC ;
SIGNAL d_out_appl_int                         : STD_LOGIC_VECTOR(31 DOWNTO 0) ;

-- from cpp_rules_programmer
-- Insert Signals here

-- from statistics
SIGNAL data_strobe              : STD_LOGIC ;
SIGNAL cntr_data                : STD_LOGIC_VECTOR(31 DOWNTO 0) ;

-- to wrappers
SIGNAL tca_appl_in_int, dataen_appl_in_int      : STD_LOGIC ;
SIGNAL sof_appl_in_int, eof_appl_in_int         : STD_LOGIC ;
SIGNAL sop_appl_in_int, soip_appl_in_int        : STD_LOGIC ;
SIGNAL d_appl_in_int                            : STD_LOGIC_VECTOR(31 DOWNTO 0) ;

BEGIN

-------------------------------------------------------------------------------
-- Process Name: inoutflops
-- Process Sensitive To: clk
-- Process Description: This process defines 12 flip-flops coming from and
--  going to the wrappers.
-------------------------------------------------------------------------------
inoutflops: PROCESS(clk)
BEGIN
  IF (clk = '1' AND clk'EVENT) THEN
    IF (reset_l = '0') THEN
      -- from wrappers
      tca_out_appl              <= '0' ;
      d_out_appl_int            <= x"00000000" ;
      dataen_out_appl_int       <= '0' ;
      sof_out_appl_int          <= '0' ;
      soip_out_appl_int         <= '0' ;
      eof_out_appl_int          <= '0' ;
      sop_out_appl_int          <= '0' ;
      -- to wrappers
      tca_appl_in_int           <= '0' ;
      D_APPL_IN                 <= x"00000000" ;
      DataEn_APPL_IN            <= '0' ;
      SOF_APPL_IN               <= '0' ;
      SOIP_APPL_IN              <= '0' ;
      EOF_APPL_IN               <= '0' ;
      SOP_APPL_IN               <= '0' ;      
    ELSE
      -- from wrappers
      tca_out_appl              <= tca_out_appl_int ;
      d_out_appl_int            <= D_OUT_APPL ;
      dataen_out_appl_int       <= DataEn_OUT_APPL ;
      sof_out_appl_int          <= SOF_OUT_APPL ;
      soip_out_appl_int         <= SOIP_OUT_APPL ;
      eof_out_appl_int          <= EOF_OUT_APPL ;
      sop_out_appl_int          <= SOP_OUT_APPL ;
      -- to wrappers
      tca_appl_in_int           <= TCA_APPL_IN ;
      D_APPL_IN                 <= d_appl_in_int ;
      DataEn_APPL_IN            <= dataen_appl_in_int ;
      SOF_APPL_IN               <= sof_appl_in_int ;
      SOIP_APPL_IN              <= soip_appl_in_int ;
      EOF_APPL_IN               <= eof_appl_in_int ;
      SOP_APPL_IN               <= sop_appl_in_int ;
    END IF ;
  END IF ;   
END PROCESS inoutflops ;


-------------------------------------------------------------------------------
-- Port Maps
-------------------------------------------------------------------------------

crp: cpp_rules_programmer
  PORT MAP (
    clk                         => clk,
    reset_l                     => reset_l,
    -- from wrappers
    dataen_out_appl_int         => dataen_out_appl_int,
    sof_out_appl_int            => sof_out_appl_int,
    soip_out_appl_int           => soip_out_appl_int,
    eof_out_appl_int            => eof_out_appl_int,
    sop_out_appl_int            => sop_out_appl_int,
    d_out_appl_int              => d_out_appl_int,
    tca_out_appl_int            => tca_out_appl_int

    -- You will need to add your signals in here
    );

alert: alert_generator
  PORT MAP (
    clk                 => clk,
    reset_l             => reset_l,

    -- from stat block
    counter_data        => cntr_data,
    counter_data_strobe => data_strobe,

    -- to wrappers
    tca_appl_in_int     => tca_appl_in_int,
    d_appl_in_int       => d_appl_in_int,
    dataen_appl_in_int  => dataen_appl_in_int,
    sof_appl_in_int     => sof_appl_in_int,
    soip_appl_in_int    => soip_appl_in_int,
    eof_appl_in_int     => eof_appl_in_int,
    sop_appl_in_int     => sop_appl_in_int
    );


-------------------------------------------------------------------------------
-- Just for Now, we will use these signals in Machine Problem 2
-------------------------------------------------------------------------------
SRAM_Addr_dev0  <= (OTHERS => 'Z') ;
SRAM_gw_dev0    <= '0' ;
SRAM_ce_dev0    <= '0' ;
SRAM_pin84_dev0 <= '0' ;
SRAM_Addr_dev1  <= (OTHERS => 'Z') ;
SRAM_gw_dev1    <= '0' ;
SRAM_ce_dev1    <= '0' ;
SRAM_pin84_dev1 <= '0' ;


END sa ;

